// OpenMath locator interface for parse events.

package fr.inria.openmath.omapi;


/**
 * Interface for associating an Open-Math parsing event with a
 * location in imput.
 *
 * <p>If an OpenMath parser provides location information to the
 * application, it does so by implementing this interface and then
 * passing an instance to the application using the
 * handler's <code>setLocator()</code> method.  The application can use the
 * object to obtain the location of any event
 * in the source document.</p>
 *
 * <p>Note that the results returned by the object will be valid only
 * during the scope of each handler method: the application
 * will receive unpredictable results if it attempts to use the
 * locator at any other time.</p>
 *
 * <p>OpenMath parsers are not required to supply a locator, but they are
 * strongly encouraged to do so. If the parser supplies a
 * locator, it must do so before reporting any other parsing events.
 * If no locator has been set by the time the application receives
 * the <code>startParse</code> event, the application should assume that a
 * locator is not available.</p>
 *
 * @see fr.inria.openmath.omapi.ParserHandler#setLocator
 */
public interface Locator {

    /**
     * Returns the line number where the current parsing event ends.
     *
     * <p>Note that this is the line position of the first character
     * after the text associated with the event.
     * @return The line number, or -1 if none is available.
     * @see #getColumnNumber
     */
    public int getLineNumber() ;


    /**
     * Returns the column number where the current parsing event ends.
     *
     * <p>Note that this is the column number of the first
     * character after the text associated with the event.
     * The first column in a line is position 1.
     * @return The column number, or -1 if none is available.
     * @see #getLineNumber
     */
    public int getColumnNumber() ;

}
