// OpenMath base ParserAdapter class.
// $Id:

package fr.inria.openmath.omapi;

import java.math.BigInteger;

/**
 * Default base class for ParserHandlers.
 *
 * <p>This class implements the default behaviour for ParserHandler.</p>
 *
 * <p>Application writers can extend this class when they need to
 * implement only part of an interface; parser writers can
 * instantiate this class to provide default handlers when the
 * application has not supplied its own.</p>
 *
 * <p>Note that the use of this class is optional.</p>
 *
 * @see fr.inria.openmath.omapi.ParserHandler
 */
public class ParserAdapter implements ParserHandler {

    /**
     * Receives a Locator for parse events.
     *
     * <p>By default, do nothing. Application writers may override this
     * method in a subclass if they wish to store the locator for use
     * with other  events.</p>
     *
     * @param locator A locator for all OpenMath parse events.
     * @see fr.inria.openmath.omapi.ParserHandler#setLocator
     * @see fr.inria.openmath.omapi.Locator
     */
    public void setLocator(Locator locator) {} // no op

    /**
     * Receives notification of the beginning of parse (of a set of OpenMath-objects).
     *
     * <p>By default, do nothing. Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     */
    public void startParse() throws OMException {} // no op

    /**
     * Receives notification of the end of parse.
     *
     * <p>By default, do nothing. Application writers may override this
     * method in a subclass.</p>
     *
     * <p>The OpenMath parser will invoke this method only once, and it will
     * be the last method invoked during the parse.  The parser shall
     * not invoke this method until it has either abandoned parsing
     * (because of an unrecoverable error) or reached the end of
     * the input (no more OpenMath objects).</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     */
    public void endParse() throws OMException {} // no op

    /**
     * Receives notification of the beginning of a object element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     * @see #endObject
     */
    public void startObject() throws OMException {} // no op

    /**
     * Receives notification of the end of an object element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     * @see #startObject
     */
    public void endObject() throws OMException {} // no op

    /**
     * Receives notification of the beginning of a application element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     * @see #endApplication
     */
    public void startApplication() throws OMException {} // no op

    /**
     * Receives notification of the end of an application element.
     *
     * <p>By default, do nothing. Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     * @see #startApplication
     */
    public void endApplication() throws OMException {} // no op

    /**
     * Receives notification of the beginning of a attribution element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     * @see #endAttribution
     */
    public void startAttribution() throws OMException {} // no op

    /**
     * Receives notification of the end of an attribution element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     * @see #startAttribution
     */
    public void endAttribution() throws OMException {} // no op

    /**
     * Receives notification of the beginning of an attribute-pairs element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     *
     * @see #endAttributePairs
     */
    public void startAttributePairs() throws OMException {} // no op

    /**
     * Receives notification of the end of an attribute-pairs element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     * @see #startAttribution
     */
    public void endAttributePairs() throws OMException {} // no op

    /**
     * Receives notification of the beginning of a Bind element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     *
     * @see #endBind
     */
    public void startBind() throws OMException {} // no op

    /**
     * Receives notification of the beginning of a BVars element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     *
     * @see #endBVars
     */
    public void startBVars() throws OMException {} // no op

    /**
     * Receives notification of the beginning of a Bind element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     *
     * @see #endBind
     */
    public void endBind() throws OMException {} // no op

    /**
     * Receives notification of the beginning of a BVars element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     *
     * @see #startBVars
     */
    public void endBVars() throws OMException {} // no op

    /**
     * Receives notification of the beginning of a error element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     *
     * @see #endError
     */
    public void startError() throws OMException {} // no op

    /**
     * Receives notification of the end of an error element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     * @see #startError
     */
    public void endError() throws OMException {} // no op

    /**
     * Receives notification of an Integer element.
     *
     * <p>By default, do nothing. Application writers may override this
     * method in a subclass.</p>
     *
     * @param value the value embedded in this Integer element.
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     */
    public void readInteger(int value) throws OMException {} // no op

    /**
     * Receives notification of a Big Integer element.
     *
     * <p>By default, do nothing. Application writers may override this
     * method in a subclass.</p>
     *
     * @param value the value embedded in this Integer element. This is an infinite precision integer.
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     */
    public void readInteger(BigInteger value) throws OMException {} // no op

    /**
     * Receives notification of a floating-point number element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @param value the value embedded in this Float element.
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     *
     */
    public void readFloat(double value) throws OMException {} // no op

    /**
     * Receives notification of a string element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @param value the value embedded in this String element.
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     *
     */
    public void readString(String value) throws OMException {} // no op

    /**
     * Receives notification of a variable element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @param name the name of this Variable element.
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     *
     */
    public void readVariable(String name) throws OMException {} // no op

    /**
     * Receives notification of a byteArray element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @param value the value embedded in this ByteArray element.
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     *
     */
    public void readByteArray(byte[] value) throws OMException {} // no op

    /**
     * Receives notification of a symbol element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @param value the value embedded in this Symbol element.
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     * @see Symbol
     */
    public void readSymbol(Symbol value) throws OMException {} // no op

    /**
     * Receives notification of a comment element.
     *
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass.</p>
     *
     * @param value the value embedded in this Comment element.
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     * @see Comment
     */
    public void readComment(String value) throws OMException {} // no op

    /**
     * Receives notification of a processing instruction.
     *
     * <p>The Parser will invoke this method each time a processing instruction
     *  is parsed.</p>
     *
     * @param value the value embedded in this Processing Instruction element.
     * @exception fr.inria.openmath.omapi.OMException Any OpenMath exception.
     */
    public void readPInstruction(String value) throws OMException {} // no op

}
