// A demo/test programm for OpenMath parser/printers interfaces/classes

import fr.inria.openmath.omapi.*;
import fr.inria.openmath.omapi.implementation.*;
import java.io.*;

/* This is a simple demo of INRIA's omapi and 
 * omapi.implementaion.
 * Here we parse the OpenMath objects coming on a stream
 * and we print them back to another stream. Thus we mimic
 * a pipe.
 * The parse is done by a Parser that process incoming 
 * data and calls methods of an object implementing the 
 * ParserHandler interface.
 * Thus you only have to chose a Parser implementation,
 * plug a ParserHandler of yours and start parse.
 * Each time a parse event occurs the parser calls the
 * specific processing code that is hold in the 
 * ParserHandler.
 *   eg: when a OMVAR token is found the correspondig
 *       method readVariable of the ParserHanclder is 
 *       called and you can do what you want with the
 *       variable content.
 * Here we use a predefined ParserHamdler that print 
 * each given token back, but it's up to you to build 
 * new ParserHandlers.
 *
 * One more word about printing OpenMath objects.
 * Printers are objects implementing the 
 * fr.inria.omapi.Printer interface.
 * This interface provides caller with all methods needed
 * to serialize OpenMath objects. The implementation 
 * package contains at least three printers 
 * (XML/binary/human readable...). Thus objects may be
 * serialized in various formats.
 */

class Pipe {

    static InputStream in;

    static OutputStream out;

    static Parser parser;

    static ParserHandler pipeParserHandler;

    static Printer printer;

    static String inFileName, outFileName;

    private void usage() {
        System.out.println("usage:\n" +
                "java Pipe -{x|b}{x|b}[d] infile outfile\n" +
                "  demo programm for openmath parser/printer classes.\n" +
                "  it pipes the content of 'infile' into 'outfile'.\n" +
                "  x/b means xml/binary encoding.\n" +
                "  eg: java Pipe -xb test.omt test.omb\n" +
                "  converts the xml encoded OpenMath objects in file test.omt\n"
                + "  into binary encoded objects stored into file test.omb.\n"
                + "  d flag may be added to trace parsing.\n" +
                "  eg: java Pipe -xxd test.omt pretty.omt\n" +
                "  parse test.omt with trace enabled and reprint it into pretty.omt."
                );
        System.exit(1);
    }

    private void parseArgs(String args[]) {
        char f = ' ', t = ' ', d = ' ';

        try {
            if ((args.length == 0) || !(args[0].startsWith("-"))) {
                usage();
            }

            f = args[0].charAt(1);
            t = args[0].charAt(2);
            if ((args[0]).length() > 3) {
                d = args[0].charAt(3);
            }
            if (args.length >= 2) {
                inFileName = args[1];
            } else {
                inFileName = "test.omt";
            }
            if (args.length >= 3) {
                outFileName = args[2];
            } else {
                outFileName = "test.todel";
            }
        } catch (Exception e) {
            System.out.println("Something went wrong while parsing options.\n"
                    + e);
            usage();
        }

        try {
            // Output will be directed to this file.
            out = new DataOutputStream(
                    new BufferedOutputStream(
                    new FileOutputStream(new File(outFileName))));

            // input data stream 
            in = new DataInputStream(
                    new BufferedInputStream(
                    new FileInputStream(new File(inFileName))));

        } catch (Exception e) {
            System.out.println(
                    "Something went wrong while opening I/O files.\n" + e);
            usage();
        }

        if (t == 'x') {
            // Here, we print tokens with the XML encoding.
            printer = new XMLPrinter(out);
        } else if (t == 'b') {
            // Print tokens with the Binary encoding.
            printer = new BinaryPrinter(out);
        } else {
            usage();
        }

        // This is the ParserHandler that will be
        // used by the main parser
        // This one is a pipe (ie: it prints back
        // each given token)
        if (d == 'd') {
            // This one trace parsing
            pipeParserHandler =
                    new DebugParserHandler(new PipeParserHandler(printer));
        } else {
            pipeParserHandler = new PipeParserHandler(printer);
        }

        if (f == 'x') {
            // This one will try to parse XML
            parser = new XMLParser(pipeParserHandler);
        } else if (f == 'b') {
            // This one will try to parse Binary
            parser = new BinaryParser(pipeParserHandler);
        } else {
            usage();
        }

    }

    public static void main(String args[]) {
        Pipe pipe = new Pipe();
        pipe.parseArgs(args);
        try {
            // Setup done, now call the parser main loop.
            pipe.parser.initParse(in);
            pipe.parser.parse(in);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
