// A demo/test programm for OpenMath tree specific interfaces/classes

import fr.inria.openmath.omapi.*;
import fr.inria.openmath.omapi.implementation.*;
import java.io.*;

/* This is a more sophisticated demo code. (You should 
 * probably look at Pipe.java first.)
 * It does a parse like Pipe but it also builds trees for 
 * parsed OpenMath objects.
 * When a tree has been completly build (and thus checked)
 * it is printed by a TreePrinter.
 *
 * The parse is done like in Pipe but the ParserHandler
 * does not pipe tokens as soon as they are read. Instead
 * it builds a tree of Node objects. Thus when the parse
 * of a whole OM object is finished it gives the whole tree
 * to the parsedTree method of an object implementing the
 * TreeHandler interface.
 * Here, this method gives the tree back to a TreePrinter 
 * that serialize it.
 */

public class TreePipe implements TreeHandler {

    static TreePrinterImpl treePrinter;

    public void parsedTree(Node root) throws OMException {
        // In this demo we use a ParserHandler that builds
        // trees and calls the parsedTree method of another
        // handler (TreeHandler) when a tree is ready.
        // Thus this method is called each time a new tree
        // has been build.
        treePrinter.printTree(root);   // serialize tree
    }

    public static void main(String args[]) {
        OutputStream out;
        InputStream in;
        Parser parser;
        TreeParserHandler treeParserHandler;
        ParserHandler debugParserHandler;
        TreePipe treePipe;
        String inFileName, outFileName;

        if (args.length >= 1) {
            inFileName = args[0];
        } else {
            inFileName = "test.in";
        }
        if (args.length >= 2) {
            outFileName = args[1];
        } else {
            outFileName = "test.out";
        }


        try {
            // Output will be directed to this file.
            out = new DataOutputStream(
                    new BufferedOutputStream(
                    new FileOutputStream(new File(outFileName))));
            // A tree printer is able to serialize a tree
            // using a generic OpenMath Printer interface.
            // This one uses a Human Printer that will 
            // print OpenMath objects in a human readable
            // form (less verbose than XML encoding)
            treePrinter = new TreePrinterImpl(new HumanPrinter(out));

            // A TreeParserHandler is a ParserHandler that
            // builds a tree of parsed OM objects.
            treeParserHandler = new TreeParserHandler();

            // We need an object that implements the 
            // TreeHandler interface called by 
            // TreeParserHandler. For the sake of 
            // simplicity I use this TreePipe class.
            treePipe = new TreePipe();
            treeParserHandler.setTreeHandler(treePipe);

            // We need a parser to read incoming stream.
            // INRIA's implementation provides you with
            // XML or binary parsers
            // A ParserHandler is given to the parser
            // the methods of the ParserHandler are called
            // at each parse event.
            parser = new XMLParser(treeParserHandler);

            // You may want to use a DebugParserHandler to
            // trace events. A DebugParserHandler can 
            // perform the trace AND call another
            // ParserHandler. Thus cascading a 'debug' and
            // a 'tree' parser handler is an easy way to 
            // look at the events going to the real 'tree' 
            // handler.
            //            debugParserHandler = new DebugParserHandler(treeParserHandler);
            //            parser = new XMLParser(debugParserHandler);

            // incoming stream 
            in = new DataInputStream(
                    new BufferedInputStream(
                    new FileInputStream(new File(inFileName))));

            // Calls the parser mainloop.
            parser.initParse(in);
            parser.parse(in);

            // The treePrinter that is called from the
            // TreeHandler need to know when we consider
            // print has 'ended'.
            // Thus we must call its endPrint() explicitly.
            treePrinter.endPrint();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
